from flask import Flask, render_template, request, jsonify, redirect, url_for, session, flash
from flask_sqlalchemy import SQLAlchemy
import os
import pandas as pd
import joblib  # Ensure joblib is imported
import numpy as np  # Ensure numpy is imported
import csv
from werkzeug.utils import secure_filename
from werkzeug.security import generate_password_hash, check_password_hash
from functools import wraps
from reportlab.pdfgen import canvas
import io
from reportlab.lib.pagesizes import A4
from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import SimpleDocTemplate, Table, TableStyle, Paragraph
from datetime import datetime
from flask import send_file
from flask import send_from_directory
from crypto_utils import encrypt_value, decrypt_value

model = joblib.load("dietitian_referral_model.pkl")
scaler = joblib.load("scaler.pkl")

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = 'mysql://root:Immortal10!!@localhost/hospital_db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.secret_key = 'a_more_persistent_secret_key'

db = SQLAlchemy(app)


# Define the doctors table model
class Doctor(db.Model):
    __tablename__ = 'doctors'
    doctor_id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.String(50))
    last_name = db.Column(db.String(50))
    email = db.Column(db.String(100))
    password = db.Column(db.String(255))
    specialty = db.Column(db.String(100))

class FeedingDashboardData(db.Model):
    __tablename__ = 'feeding_dashboard_data'
    encounterId = db.Column(db.Integer, primary_key=True)
    end_tidal_co2 = db.Column(db.Text)
    feed_vol = db.Column(db.Text)
    feed_vol_adm = db.Column(db.Text)
    fio2 = db.Column(db.Text)
    fio2_ratio = db.Column(db.Text)
    insp_time = db.Column(db.Text)
    oxygen_flow_rate = db.Column(db.Text)
    peep = db.Column(db.Text)
    pip = db.Column(db.Text)
    resp_rate = db.Column(db.Text)
    sip = db.Column(db.Text)
    tidal_vol = db.Column(db.Text)
    tidal_vol_actual = db.Column(db.Text)
    tidal_vol_kg = db.Column(db.Text)
    tidal_vol_spontaneous = db.Column(db.Text)
    bmi = db.Column(db.Text)
    referral = db.Column(db.Integer)
    ml_predicted_referral = db.Column(db.Integer, nullable=True)  # <-- Add this column

    
@app.route('/')
def index():
    return redirect(url_for('login'))


def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'doctor_id' not in session:
            flash("Please log in to access this page.", "info")
            return redirect(url_for('login'))
        return f(*args, **kwargs)
    return decorated_function

@app.route('/signup', methods=['GET', 'POST'])
def signup():
    if request.method == 'POST':
        try:
            first_name = request.form['first_name']
            last_name = request.form['last_name']
            email = request.form['email']
            password = request.form['password']
            specialty = request.form.get('specialty')

            # Logging form data to verify correctness
            print(f"Received form data: {first_name}, {last_name}, {email}, {specialty}")

            # Hash the password for secure storage
            hashed_password = generate_password_hash(password, method='pbkdf2:sha256')

            # Create and add the new doctor
            new_doctor = Doctor(
                first_name=first_name,
                last_name=last_name,
                email=email,
                password=hashed_password,
                specialty=specialty
            )

            db.session.add(new_doctor)
            db.session.commit()

            flash("Account created successfully! Please log in.", "success")
            print("Account created successfully")
            return redirect(url_for('login'))
        except Exception as e:
            db.session.rollback()
            print(f"Error creating account: {e}")
            flash(f"Error creating account: {e}", "danger")

    return render_template('signup.html')



@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        email = request.form['email']
        password = request.form['password']
        
        # Fetch the doctor by email
        doctor = Doctor.query.filter_by(email=email).first()
        
        # Check password hash
        if doctor and check_password_hash(doctor.password, password):
            session['doctor_id'] = doctor.doctor_id  # Store doctor ID in session
            flash("Login successful!", "success")
            return redirect(url_for('dashboard'))
        
        flash("Invalid email or password", "danger")

    return render_template('login.html')



@app.route('/dashboard')
@login_required
def dashboard():
    return render_template('dashboard.html')


@app.route('/logout')
def logout():
    session.clear()
    flash("You have been logged out.", "info")
    return redirect(url_for('login'))



@app.route('/upload_csv', methods=['POST'])
def upload_csv():
    if 'file' not in request.files:
        return jsonify({"error": "No file part in the request"}), 400

    file = request.files['file']
    if file.filename == '':
        return jsonify({"error": "No selected file"}), 400

    filename = secure_filename(file.filename)
    upload_folder = app.config.get('UPLOAD_FOLDER', 'uploads')

    if not os.path.exists(upload_folder):
        os.makedirs(upload_folder)

    filepath = os.path.join(upload_folder, filename)
    file.save(filepath)

    try:
        with open(filepath, newline='') as csvfile:
            csv_data = csv.reader(csvfile)
            next(csv_data)  # Skip header row

            for row in csv_data:
                if len(row) != 18:
                    return jsonify({"error": "CSV format is incorrect"}), 400

                # Check if encounterId already exists
                existing_entry = FeedingDashboardData.query.filter_by(encounterId=row[0]).first()
                if existing_entry:
                    print(f"Skipping duplicate entry for Encounter ID {row[0]}")
                    continue  # Skip inserting duplicate

                # Convert to float, use 0 for missing values
                feature_values = np.array([float(row[i]) if row[i] else 0.0 for i in range(1, 17)]).reshape(1, -1)

                # Apply feature names to prevent Scikit-learn warning
                feature_names = [
                    "end_tidal_co2", "feed_vol", "feed_vol_adm", "fio2", "fio2_ratio",
                    "insp_time", "oxygen_flow_rate", "peep", "pip", "resp_rate",
                    "sip", "tidal_vol", "tidal_vol_actual", "tidal_vol_kg",
                    "tidal_vol_spontaneous", "bmi"
                ]
                feature_df = pd.DataFrame(feature_values, columns=feature_names)
                # Transform using scaler
                scaled_features = scaler.transform(feature_df)
                ml_prediction = model.predict(scaled_features)[0]  # Get ML prediction

                print(f"Predicted Referral for Encounter {row[0]}: {ml_prediction}")  # ✅ Debug: Print Prediction

                # Insert only if it does not exist
                new_entry = FeedingDashboardData(
                    encounterId=row[0],
                    end_tidal_co2=row[1],
                    feed_vol=row[2],
                    feed_vol_adm=row[3],
                    fio2=row[4],
                    fio2_ratio=row[5],
                    insp_time=row[6],
                    oxygen_flow_rate=row[7],
                    peep=row[8],
                    pip=row[9],
                    resp_rate=row[10],
                    sip=row[11],
                    tidal_vol=row[12],
                    tidal_vol_actual=row[13],
                    tidal_vol_kg=row[14],
                    tidal_vol_spontaneous=row[15],
                    bmi = encrypt_value(row[16]),
                    referral=row[17],
                    ml_predicted_referral=int(ml_prediction) if ml_prediction is not None else 0
                )
                db.session.add(new_entry)

            db.session.commit()
            print("✅ All ML predictions saved to database!")  # ✅ Debugging confirmation

    except Exception as e:
        print("Error:", e)
        return jsonify({"error": f"An error occurred while processing the CSV file: {str(e)}"}), 500

    return jsonify({"status": "CSV data uploaded successfully"}), 200




@app.route('/api/patient_data', methods=['GET'])
def get_patient_data():
    show_referred = request.args.get("referred", "false").lower() == "true"
    show_ml_predicted = request.args.get("ml_predicted", "false").lower() == "true"

    query = FeedingDashboardData.query

    if show_referred and not show_ml_predicted:
        # Show only referred patients (ignore ML prediction)
        query = query.filter(FeedingDashboardData.referral == 1)

    elif show_ml_predicted and not show_referred:
        # Show only ML-predicted referred patients
        query = query.filter(FeedingDashboardData.ml_predicted_referral == 1)

    elif show_referred and show_ml_predicted:
        # Show both referred & ML-predicted patients
        query = query.filter(
            (FeedingDashboardData.referral == 1) | (FeedingDashboardData.ml_predicted_referral == 1)
        )

    data = query.all()

    # Modify response to include a "highlight" flag
    return jsonify([
        {
            "encounterId": entry.encounterId,
            "end_tidal_co2": entry.end_tidal_co2,
            "feed_vol": entry.feed_vol,
            "feed_vol_adm": entry.feed_vol_adm,
            "fio2": entry.fio2,
            "fio2_ratio": entry.fio2_ratio,
            "insp_time": entry.insp_time,
            "oxygen_flow_rate": entry.oxygen_flow_rate,
            "peep": entry.peep,
            "pip": entry.pip,
            "resp_rate": entry.resp_rate,
            "sip": entry.sip,
            "tidal_vol": entry.tidal_vol,
            "tidal_vol_actual": entry.tidal_vol_actual,
            "tidal_vol_kg": entry.tidal_vol_kg,
            "tidal_vol_spontaneous": entry.tidal_vol_spontaneous,
            "bmi": decrypt_value(entry.bmi) if entry.bmi.strip() else "N/A",
            "referral": entry.referral,
            "ml_predicted_referral": entry.ml_predicted_referral,
            "highlight": (entry.ml_predicted_referral == 1 and show_ml_predicted)  # Highlight only when ML checkbox is on
        }
        for entry in data
    ])





@app.route('/api/generate_graphical_report')
def generate_graphical_report_data():
    data = FeedingDashboardData.query.all()
    # Format data for graphical report
    response_data = [{
        "encounterId": entry.encounterId,
        "end_tidal_co2": entry.end_tidal_co2,
        "feed_vol": entry.feed_vol,
        "feed_vol_adm": entry.feed_vol_adm,
        "fio2": entry.fio2,
        "fio2_ratio": entry.fio2_ratio,
        "insp_time": entry.insp_time,
        "oxygen_flow_rate": entry.oxygen_flow_rate,
        "peep": entry.peep,
        "pip": entry.pip,
        "resp_rate": entry.resp_rate,
        "sip": entry.sip,
        "tidal_vol": entry.tidal_vol,
        "tidal_vol_actual": entry.tidal_vol_actual,
        "tidal_vol_kg": entry.tidal_vol_kg,
        "tidal_vol_spontaneous": entry.tidal_vol_spontaneous,
        "bmi": entry.bmi
    } for entry in data]
    
    return jsonify(response_data)

@app.route('/generate_summary_pdf', methods=['GET'])
def generate_summary_pdf():
    data = FeedingDashboardData.query.all()
    
    if not data:
        return "No data available to generate the report", 400

    # ✅ Ensure directory exists
    reports_folder = os.path.join(app.root_path, "static", "reports")
    os.makedirs(reports_folder, exist_ok=True)  # Creates folder if missing

    pdf_filename = "patient_summary_report.pdf"
    pdf_path = os.path.join(reports_folder, pdf_filename)

    # ✅ Create PDF
    doc = SimpleDocTemplate(pdf_path, pagesize=A4)
    elements = []
    
    styles = getSampleStyleSheet()
    title = Paragraph("<b>Comprehensive Patient Summary Report</b>", styles["Title"])
    elements.append(title)
    
    date_info = Paragraph(f"Generated on: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}", styles["Normal"])
    elements.append(date_info)

    elements.append(Paragraph("<b>Summary Statistics:</b>", styles["Heading2"]))

    # ✅ Total Counts
    total_patients = len(data)
    total_referred = sum(1 for entry in data if entry.referral == 1)
    ml_referred = sum(1 for entry in data if entry.ml_predicted_referral == 1)

    summary_data = [
        ["Metric", "Value"],
        ["Total Patients", total_patients],
        ["Total Referred Patients", total_referred],
        ["ML-Predicted Referrals", ml_referred]
    ]

    summary_table = Table(summary_data)
    summary_table.setStyle(TableStyle([
        ('BACKGROUND', (0, 0), (-1, 0), colors.grey),
        ('TEXTCOLOR', (0, 0), (-1, 0), colors.whitesmoke),
        ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
        ('FONTNAME', (0, 0), (-1, 0), 'Helvetica-Bold'),
        ('BOTTOMPADDING', (0, 0), (-1, 0), 10),
        ('GRID', (0, 0), (-1, -1), 0.5, colors.grey)
    ]))

    elements.append(summary_table)

    # ✅ Add space before next section
    elements.append(Paragraph("<br/><br/>", styles["Normal"]))
    elements.append(Paragraph("<b>Average Patient Indicators:</b>", styles["Heading2"]))

    # ✅ Averages for key indicators
    indicators = [
        "end_tidal_co2", "feed_vol", "feed_vol_adm", "fio2", "fio2_ratio",
        "insp_time", "oxygen_flow_rate", "peep", "pip", "resp_rate",
        "sip", "tidal_vol", "tidal_vol_actual", "tidal_vol_kg",
        "tidal_vol_spontaneous", "bmi"
    ]

    avg_data = [["Indicator", "Average"]]
    
    for indicator in indicators:
        values = []
        for entry in data:
            try:
                raw_value = getattr(entry, indicator, 0) or 0
                if indicator == "bmi":
                    decrypted_value = decrypt_value(raw_value)
                    values.append(float(decrypted_value))
                else:
                    values.append(float(raw_value))
            except Exception as e:
                print(f"⚠️ Error with indicator '{indicator}' for encounter {entry.encounterId}: {e}")
                values.append(0.0)  # default fallback if something goes wrong
        avg_value = round(sum(values) / len(values), 2) if values else 0
        avg_data.append([indicator.replace("_", " ").title(), avg_value])


    avg_table = Table(avg_data)
    avg_table.setStyle(TableStyle([
        ('BACKGROUND', (0, 0), (-1, 0), colors.lightgrey),
        ('TEXTCOLOR', (0, 0), (-1, 0), colors.black),
        ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
        ('FONTNAME', (0, 0), (-1, 0), 'Helvetica-Bold'),
        ('BOTTOMPADDING', (0, 0), (-1, 0), 8),
        ('GRID', (0, 0), (-1, -1), 0.5, colors.black)
    ]))

    elements.append(avg_table)

    # ✅ Build PDF
    doc.build(elements)

    # ✅ Return file as download
    return send_file(pdf_path, as_attachment=True, mimetype='application/pdf')






if __name__ == '__main__':
    # Create tables if they don't exist
    with app.app_context():
        db.create_all()
    app.run(debug=True)