document.addEventListener('DOMContentLoaded', () => {
    const uploadBtn = document.getElementById('upload-btn');
    const csvUploadInput = document.getElementById('csv-upload');
    const searchInput = document.getElementById('search-input');
    const showReferredCheckbox = document.getElementById('show-referred');
    const generateGraphicalReportBtn = document.querySelector('.reports button:nth-child(2)');
    const generateSummaryReportBtn = document.querySelector('.reports button:nth-child(3)'); 

    let patientsData = [];
    let displayedData = []; // New variable to keep track of displayed data separately

    // When the button is clicked, trigger the hidden file input
    // ✅ Only handle Upload CSV button click
    uploadBtn.addEventListener('click', () => {
        csvUploadInput.click();
    });

    // ✅ These should be outside, not nested
    generateGraphicalReportBtn.addEventListener('click', generateGraphicalReport);
    generateSummaryReportBtn.addEventListener('click', generateSummaryReport);

    // ✅ Only run if login button exists
    const loginButton = document.getElementById('login-button');
    if (loginButton) {
        loginButton.onclick = function () {
            window.location.href = '/login';
        };
    }

    csvUploadInput.addEventListener('change', (event) => {
        const file = event.target.files[0];
        if (file) {
            // ✅ Show the spinner
            document.getElementById('upload-spinner').style.display = 'block'; // ✅
    
            const formData = new FormData();
            formData.append('file', file);
    
            fetch('/upload_csv', {
                method: 'POST',
                body: formData
            })
            .then(response => response.json())
            .then(data => {
                // ✅ Hide the spinner
                document.getElementById('upload-spinner').style.display = 'none';

    
                if (data.status === "CSV data uploaded successfully") {
                    fetchDataFromDatabase();
                    showUploadConfirmation();
                } else {
                    showError("Error: CSV upload failed.");
                }
            })
            .catch(error => {
                // ✅ Hide the spinner on error too
                document.getElementById('upload-spinner').style.display = 'block';
                console.error('Error uploading file:', error);
                showError("Error: Could not upload CSV.");
            });
        }
    });
    
    
    

    // Fetch data from the database
    let originalPatientsData = []; // Store unmodified original data

    function fetchDataFromDatabase() {
        fetch('/api/patient_data')
            .then(response => response.json())
            .then(data => {
                patientsData = data;
                originalPatientsData = JSON.parse(JSON.stringify(data)); // Deep copy
                displayedData = [...patientsData]; 
                
                displayCSVContents(displayedData);  // ✅ Show table updates
                updateSummaryBoxes();  // ✅ Update registered patients & referrals
            })
            .catch(error => console.error('Error fetching data:', error));
    }
    
    
    function showUploadConfirmation() {
        // Create the notification container
        const confirmationDiv = document.createElement("div");
        confirmationDiv.id = "upload-confirmation";
        confirmationDiv.innerHTML = `
            <div class="confirmation-content">
                <svg width="40" height="40" viewBox="0 0 24 24" fill="none" stroke="green" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                    <circle cx="12" cy="12" r="10"></circle>
                    <path d="M9 12l2 2 4-4"></path>
                </svg>
                <div>
                    <h3>CSV Uploaded Successfully</h3>
                    <p>Your data has been processed and added to the system.</p>
                </div>
            </div>
        `;
    
        confirmationDiv.classList.add("upload-confirmation-box");
        document.body.appendChild(confirmationDiv);
    
        // ✅ Fetch and update data in the background
        fetchDataFromDatabase();
    
        // ❌ Remove the confirmation box smoothly after 6 seconds
        setTimeout(() => {
            confirmationDiv.classList.add("fade-out"); // Start fade out animation
            setTimeout(() => confirmationDiv.remove(), 500); // Remove after animation
        }, 6000);
    }
    
    
// ❌ Function to show a nice error pop-up if upload fails
    function showError(message) {
        Swal.fire({
            title: "Upload Failed",
            text: message,
            icon: "error",
            confirmButtonText: "Try Again",
            showCloseButton: true,
        });
    }


    // Listen for changes in search input and checkbox for filtering
    searchInput.addEventListener('input', filterTable);
    showReferredCheckbox.addEventListener('change', filterTable);

    // Display data in the table with click event for each row
    function displayCSVContents(data) {
        const tableBody = document.querySelector('#patient-table tbody');
        tableBody.innerHTML = '';  // Clear previous rows
    
        data.forEach(entry => {
            const row = tableBody.insertRow();
    
            // ✅ Highlight ML-referred patients **only when ML is selected**
            if (document.getElementById('show-ml-referred').checked && entry.ml_predicted_referral == 1) {
                row.style.backgroundColor = "#ffeeba"; // Light yellow highlight
                row.style.fontWeight = "bold";
            }
    
            row.insertCell(0).textContent = entry.encounterId;
            row.insertCell(1).textContent = entry.end_tidal_co2;
            row.insertCell(2).textContent = entry.feed_vol;
            row.insertCell(3).textContent = entry.feed_vol_adm;
            row.insertCell(4).textContent = entry.fio2;
            row.insertCell(5).textContent = entry.fio2_ratio;
            row.insertCell(6).textContent = entry.insp_time;
            row.insertCell(7).textContent = entry.oxygen_flow_rate;
            row.insertCell(8).textContent = entry.peep;
            row.insertCell(9).textContent = entry.pip;
            row.insertCell(10).textContent = entry.resp_rate;
            row.insertCell(11).textContent = entry.sip;
            row.insertCell(12).textContent = entry.tidal_vol;
            row.insertCell(13).textContent = entry.tidal_vol_actual;
            row.insertCell(14).textContent = entry.tidal_vol_kg;
            row.insertCell(15).textContent = entry.tidal_vol_spontaneous;
            row.insertCell(16).textContent = entry.bmi;
            row.insertCell(17).textContent = entry.referral;
    
            // ✅ Fix: Attach event listener to each row
            row.addEventListener("click", function() {
                displayPatientData(entry);  // Show patient details when clicking row
            });
        });
    }
    
    
    function displayPatientData(patient) {
        const patientDataSection = document.querySelector('.patient-data');
        patientDataSection.innerHTML = `
            <h2>Patient Data:</h2>
            <p>Encounter ID: ${patient.encounterId}</p>
            <p>End Tidal CO2: ${patient.end_tidal_co2}</p>
            <p>Feed Volume: ${patient.feed_vol}</p>
            <p>Feed Volume Administered: ${patient.feed_vol_adm}</p>
            <p>FiO2: ${patient.fio2}</p>
            <p>FiO2 Ratio: ${patient.fio2_ratio}</p>
            <p>Inspiration Time: ${patient.insp_time}</p>
            <p>Oxygen Flow Rate: ${patient.oxygen_flow_rate}</p>
            <p>PEEP: ${patient.peep}</p>
            <p>PIP: ${patient.pip}</p>
            <p>Respiratory Rate: ${patient.resp_rate}</p>
            <p>SIP: ${patient.sip}</p>
            <p>Tidal Volume: ${patient.tidal_vol}</p>
            <p>Tidal Volume Actual: ${patient.tidal_vol_actual}</p>
            <p>Tidal Volume per KG: ${patient.tidal_vol_kg}</p>
            <p>Tidal Volume Spontaneous: ${patient.tidal_vol_spontaneous}</p>
            <p>BMI: ${patient.bmi}</p>
            <p>Referral: ${patient.referral}</p>
        `;
    
        // Call generateGraph with the patient data to render the chart
        generateGraph([
            patient.encounterId, 
            patient.end_tidal_co2, 
            patient.feed_vol, 
            patient.feed_vol_adm, 
            patient.fio2, 
            patient.fio2_ratio, 
            patient.insp_time, 
            patient.oxygen_flow_rate, 
            patient.peep, 
            patient.pip, 
            patient.resp_rate, 
            patient.sip, 
            patient.tidal_vol, 
            patient.tidal_vol_actual, 
            patient.tidal_vol_kg, 
            patient.tidal_vol_spontaneous, 
            patient.bmi
        ]);
    }
    

    // Function to update the summary boxes with counts of unique registered patients and referrals
    function updateSummaryBoxes() {
        const uniqueEncounters = new Set();
        const uniqueReferredEncounters = new Set();
    
        patientsData.forEach(patient => {
            const encounterId = patient.encounterId;
            uniqueEncounters.add(encounterId); // ✅ Count unique patients
    
            // ✅ Check if the patient is referred (either manually or ML-predicted)
            if (patient.referral == 1 || patient.referral == "1.0" || patient.ml_predicted_referral == 1) {
                uniqueReferredEncounters.add(encounterId);
            }
        });
    
        // ✅ Update the counts dynamically
        document.getElementById('registered-patients-count').textContent = uniqueEncounters.size;
        document.getElementById('referrals-count').textContent = uniqueReferredEncounters.size;
    }
    
    const showMLReferredCheckbox = document.getElementById('show-ml-referred');
    showMLReferredCheckbox.addEventListener('change', filterTable);


    // Function to filter the table based on search input and checkbox
    function filterTable() {
        const searchValue = searchInput.value.toLowerCase();
        const showReferred = showReferredCheckbox.checked;
        const showMLReferred = document.getElementById('show-ml-referred').checked;
    
        if (showMLReferred) {
            // When ML Referral is selected, filter only ML-predicted referrals
            displayedData = originalPatientsData.filter(entry => entry.ml_predicted_referral == 1);
            
            // Change referral column to 1 for these patients
            displayedData.forEach(entry => {
                entry.referral = 1; // Force referral to 1 when ML is selected
            });
        } else {
            // Restore original data when ML checkbox is unchecked
            displayedData = JSON.parse(JSON.stringify(originalPatientsData)); // Reset to original CSV data
        }
    
        // Apply search and referral filters
        displayedData = displayedData.filter(entry => {
            const matchesSearch = entry.encounterId.toString().toLowerCase().includes(searchValue);
            const isReferred = entry.referral == 1 || entry.referral == "1.0";
    
            if (showReferred) {
                return matchesSearch && isReferred;
            }
            return matchesSearch;
        });
    
        displayCSVContents(displayedData);
    }    
    
    
    
    
});

// Function to generate a graph for the selected patient
function generateGraph(patient) {
    // Prepare data for the graph
    const labels = [
        'End Tidal CO2', 'Feed Volume', 'Feed Volume Administered', 'FiO2', 'FiO2 Ratio',
        'Inspiration Time', 'Oxygen Flow Rate', 'PEEP', 'PIP', 'Respiratory Rate', 'SIP',
        'Tidal Volume', 'Tidal Volume Actual', 'Tidal Volume per KG', 'Tidal Volume Spontaneous', 'BMI'
    ];

    const values = patient.slice(1, 17).map(value => parseFloat(value)); // Convert string to float for graph data

    // Generate chart container if it doesn't exist
    let chartContainer = document.getElementById('chartContainer');

    if (!chartContainer) {
        const chartDiv = document.createElement('div');
        chartDiv.id = 'chartContainer';
        chartDiv.innerHTML = '<canvas id="patientChart"></canvas>';
        document.querySelector('.main-content').appendChild(chartDiv);  // Add it under main content
    } else {
        // If the chart already exists, destroy it
        if (window.patientChart) {
            window.patientChart.destroy();
        }
    }

    const ctx = document.getElementById('patientChart').getContext('2d');

    // Create a new chart and store it in the window object
    window.patientChart = new Chart(ctx, {
        type: 'bar',
        data: {
            labels: labels,
            datasets: [{
                label: `Patient: ${patient[0]}`, // Corrected chart label
                data: values,
                backgroundColor: 'rgba(75, 192, 192, 0.2)',
                borderColor: 'rgba(75, 192, 192, 1)',
                borderWidth: 1
            }]
        },
        options: {
            responsive: true,
            maintainAspectRatio: false,
            scales: {
                y: {
                    beginAtZero: true
                }
            }
        }
    });
}

// Function to generate a summary report for the uploaded CSV
function generateSummaryReport() {
    fetch('/api/generate_summary_report')
        .then(response => response.json())
        .then(summaryData => {
            const { total_patients, referred_patients, avg_feed_volume, avg_fio2 } = summaryData;

            // Update the summary report section in the HTML
            const summaryDiv = document.getElementById('summaryReport') || createSummaryReport();
            summaryDiv.innerHTML = `
                <h3>Summary Report</h3>
                <p>Total Patients: ${total_patients}</p>
                <p>Referred Patients: ${referred_patients}</p>
                <p>Average Feed Volume: ${avg_feed_volume} ml</p>
                <p>Average FiO2: ${avg_fio2}</p>
            `;
        })
        .catch(error => console.error('Error generating summary report:', error));
}

// Function to create a summary report div if it doesn't exist
function createSummaryReport() {
    const reportSection = document.querySelector('.reports');
    const summaryDiv = document.createElement('div');
    summaryDiv.id = 'summaryReport';
    reportSection.appendChild(summaryDiv);
    return summaryDiv;
}



function generateGraphicalReport() {
    fetch('/api/generate_graphical_report')
        .then(response => response.json())
        .then(patientsData => {
            const labels = [
                'End Tidal CO2', 'Feed Volume', 'Feed Volume Administered', 'FiO2', 'FiO2 Ratio',
                'Inspiration Time', 'Oxygen Flow Rate', 'PEEP', 'PIP', 'Respiratory Rate', 'SIP',
                'Tidal Volume', 'Tidal Volume Actual', 'Tidal Volume per KG', 'Tidal Volume Spontaneous', 'BMI'
            ];

            // Aggregate data from all patients for each metric
            const aggregatedData = Array(labels.length).fill(0);
            patientsData.forEach(patient => {
                const values = [
                    patient.end_tidal_co2, patient.feed_vol, patient.feed_vol_adm, patient.fio2,
                    patient.fio2_ratio, patient.insp_time, patient.oxygen_flow_rate, patient.peep,
                    patient.pip, patient.resp_rate, patient.sip, patient.tidal_vol, patient.tidal_vol_actual,
                    patient.tidal_vol_kg, patient.tidal_vol_spontaneous, patient.bmi
                ].map(value => parseFloat(value) || 0);

                values.forEach((val, index) => {
                    aggregatedData[index] += val;
                });
            });

            let reportChartContainer = document.getElementById('reportChartContainer');
            if (!reportChartContainer) {
                const reportChartDiv = document.createElement('div');
                reportChartDiv.id = 'reportChartContainer';
                reportChartDiv.innerHTML = '<canvas id="reportChart" style="width: 100%; height: 600px;"></canvas>';
                document.querySelector('.reports').appendChild(reportChartDiv);
            } else {
                if (window.reportChart) window.reportChart.destroy();
            }

            const ctx = document.getElementById('reportChart').getContext('2d');
            window.reportChart = new Chart(ctx, {
                type: 'bar',
                data: {
                    labels: labels,
                    datasets: [{
                        label: 'Aggregate Data for All Patients',
                        data: aggregatedData,
                        backgroundColor: 'rgba(54, 162, 235, 0.2)',
                        borderColor: 'rgba(54, 162, 235, 1)',
                        borderWidth: 1
                    }]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    scales: {
                        y: { beginAtZero: true }
                    }
                }
            });
        })
        .catch(error => console.error('Error generating graphical report:', error));
    
}

document.addEventListener('DOMContentLoaded', () => {
    document.getElementById('generate-summary-pdf-btn').addEventListener('click', () => {
        fetch('/generate_summary_pdf')
            .then(response => {
                if (!response.ok) {
                    throw new Error("Failed to generate PDF");
                }
                return response.blob();
            })
            .then(blob => {
                const url = window.URL.createObjectURL(blob);
                const a = document.createElement('a');
                a.href = url;
                a.download = 'patient_summary_report.pdf'; // ✅ Download PDF
                document.body.appendChild(a);
                a.click();
                window.URL.revokeObjectURL(url);
            })
            .catch(error => {
                console.error('Error generating PDF:', error);
            });
    });
});







const zoomMeetingBtn = document.getElementById('zoom-meeting-btn');

zoomMeetingBtn.addEventListener('click', () => {
    const zoomMeetingUrl = "https://zoom.us/join"; // Replace with your Zoom meeting link
    window.open(zoomMeetingUrl, '_blank'); // Open the Zoom link in a new tab
});