from flask import Flask, render_template, request, jsonify, redirect, url_for, session, flash
from flask_sqlalchemy import SQLAlchemy
import os
import csv
from werkzeug.utils import secure_filename
from werkzeug.security import generate_password_hash, check_password_hash
from functools import wraps

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = 'mysql://root:Am_20462046@localhost/hospital_db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.secret_key = 'a_more_persistent_secret_key'

db = SQLAlchemy(app)


# Define the doctors table model
class Doctor(db.Model):
    __tablename__ = 'doctors'
    doctor_id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.String(50))
    last_name = db.Column(db.String(50))
    email = db.Column(db.String(100))
    password = db.Column(db.String(255))
    specialty = db.Column(db.String(100))

# Define the feeding_dashboard_data table model
class FeedingDashboardData(db.Model):
    __tablename__ = 'feeding_dashboard_data'
    encounterId = db.Column(db.Integer, primary_key=True)
    end_tidal_co2 = db.Column(db.Text)
    feed_vol = db.Column(db.Text)
    feed_vol_adm = db.Column(db.Text)
    fio2 = db.Column(db.Text)
    fio2_ratio = db.Column(db.Text)
    insp_time = db.Column(db.Text)
    oxygen_flow_rate = db.Column(db.Text)
    peep = db.Column(db.Text)
    pip = db.Column(db.Text)
    resp_rate = db.Column(db.Text)
    sip = db.Column(db.Text)
    tidal_vol = db.Column(db.Text)
    tidal_vol_actual = db.Column(db.Text)
    tidal_vol_kg = db.Column(db.Text)
    tidal_vol_spontaneous = db.Column(db.Text)
    bmi = db.Column(db.Text)
    referral = db.Column(db.Integer)
    
@app.route('/')
def index():
    return redirect(url_for('login'))


def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'doctor_id' not in session:
            flash("Please log in to access this page.", "info")
            return redirect(url_for('login'))
        return f(*args, **kwargs)
    return decorated_function

@app.route('/signup', methods=['GET', 'POST'])
def signup():
    if request.method == 'POST':
        try:
            first_name = request.form['first_name']
            last_name = request.form['last_name']
            email = request.form['email']
            password = request.form['password']
            specialty = request.form.get('specialty')

            # Logging form data to verify correctness
            print(f"Received form data: {first_name}, {last_name}, {email}, {specialty}")

            # Hash the password for secure storage
            hashed_password = generate_password_hash(password, method='pbkdf2:sha256')

            # Create and add the new doctor
            new_doctor = Doctor(
                first_name=first_name,
                last_name=last_name,
                email=email,
                password=hashed_password,
                specialty=specialty
            )

            db.session.add(new_doctor)
            db.session.commit()

            flash("Account created successfully! Please log in.", "success")
            print("Account created successfully")
            return redirect(url_for('login'))
        except Exception as e:
            db.session.rollback()
            print(f"Error creating account: {e}")
            flash(f"Error creating account: {e}", "danger")

    return render_template('signup.html')



@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        email = request.form['email']
        password = request.form['password']
        
        # Fetch the doctor by email
        doctor = Doctor.query.filter_by(email=email).first()
        
        # Check password hash
        if doctor and check_password_hash(doctor.password, password):
            session['doctor_id'] = doctor.doctor_id  # Store doctor ID in session
            flash("Login successful!", "success")
            return redirect(url_for('dashboard'))
        
        flash("Invalid email or password", "danger")

    return render_template('login.html')



@app.route('/dashboard')
@login_required
def dashboard():
    return render_template('dashboard.html')


@app.route('/logout')
def logout():
    session.clear()
    flash("You have been logged out.", "info")
    return redirect(url_for('login'))




# Route to handle CSV upload and save data to the database
@app.route('/upload_csv', methods=['POST'])
def upload_csv():
    # Ensure 'file' is in the request
    if 'file' not in request.files:
        return jsonify({"error": "No file part in the request"}), 400

    file = request.files['file']
    if file.filename == '':
        return jsonify({"error": "No selected file"}), 400

    # Sanitize filename
    filename = secure_filename(file.filename)
    upload_folder = app.config.get('UPLOAD_FOLDER', 'uploads')

    # Create the upload folder if it doesn't exist
    if not os.path.exists(upload_folder):
        os.makedirs(upload_folder)

    # Save the file securely
    filepath = os.path.join(upload_folder, filename)
    file.save(filepath)

    # Try reading the CSV file and inserting data
    try:
        with open(filepath, newline='') as csvfile:
            csv_data = csv.reader(csvfile)
            next(csv_data)  # Skip header row if present

            # Process each row in the CSV
            for row in csv_data:
                if len(row) != 18:
                    return jsonify({"error": "CSV format is incorrect"}), 400

                print("Row data:", row)

                new_entry = FeedingDashboardData(
                    encounterId=row[0],
                    end_tidal_co2=row[1],
                    feed_vol=row[2],
                    feed_vol_adm=row[3],
                    fio2=row[4],
                    fio2_ratio=row[5],
                    insp_time=row[6],
                    oxygen_flow_rate=row[7],
                    peep=row[8],
                    pip=row[9],
                    resp_rate=row[10],
                    sip=row[11],
                    tidal_vol=row[12],
                    tidal_vol_actual=row[13],
                    tidal_vol_kg=row[14],
                    tidal_vol_spontaneous=row[15],
                    bmi=row[16],
                    referral=row[17]
                )
                db.session.add(new_entry)

            # Commit changes to the database
            db.session.commit()

    except Exception as e:
        print("Error:", e)
        return jsonify({"error": f"An error occurred while processing the CSV file: {str(e)}"}), 500

    # Confirm successful upload
    return jsonify({"status": "CSV data uploaded successfully"}), 200


@app.route('/api/patient_data', methods=['GET'])
def get_patient_data():
    data = FeedingDashboardData.query.all()
    return jsonify([{
        "encounterId": entry.encounterId,
        "end_tidal_co2": entry.end_tidal_co2,
        "feed_vol": entry.feed_vol,
        "feed_vol_adm": entry.feed_vol_adm,
        "fio2": entry.fio2,
        "fio2_ratio": entry.fio2_ratio,
        "insp_time": entry.insp_time,
        "oxygen_flow_rate": entry.oxygen_flow_rate,
        "peep": entry.peep,
        "pip": entry.pip,
        "resp_rate": entry.resp_rate,
        "sip": entry.sip,
        "tidal_vol": entry.tidal_vol,
        "tidal_vol_actual": entry.tidal_vol_actual,
        "tidal_vol_kg": entry.tidal_vol_kg,
        "tidal_vol_spontaneous": entry.tidal_vol_spontaneous,
        "bmi": entry.bmi,
        "referral": entry.referral
    } for entry in data])


@app.route('/api/generate_graphical_report')
def generate_graphical_report_data():
    data = FeedingDashboardData.query.all()
    # Format data for graphical report
    response_data = [{
        "encounterId": entry.encounterId,
        "end_tidal_co2": entry.end_tidal_co2,
        "feed_vol": entry.feed_vol,
        "feed_vol_adm": entry.feed_vol_adm,
        "fio2": entry.fio2,
        "fio2_ratio": entry.fio2_ratio,
        "insp_time": entry.insp_time,
        "oxygen_flow_rate": entry.oxygen_flow_rate,
        "peep": entry.peep,
        "pip": entry.pip,
        "resp_rate": entry.resp_rate,
        "sip": entry.sip,
        "tidal_vol": entry.tidal_vol,
        "tidal_vol_actual": entry.tidal_vol_actual,
        "tidal_vol_kg": entry.tidal_vol_kg,
        "tidal_vol_spontaneous": entry.tidal_vol_spontaneous,
        "bmi": entry.bmi
    } for entry in data]
    
    return jsonify(response_data)

@app.route('/api/generate_summary_report')
def generate_summary_report():
    data = FeedingDashboardData.query.all()
    
    # Initialize summary variables
    total_patients = len(data)
    referred_patients = 0
    feed_volume_sum = 0.0
    fio2_sum = 0.0

    # Calculate the required summary statistics
    for entry in data:
        if entry.referral == 1 or entry.referral == "1.0":
            referred_patients += 1
        feed_volume_sum += float(entry.feed_vol) if entry.feed_vol else 0.0
        fio2_sum += float(entry.fio2) if entry.fio2 else 0.0

    # Calculate averages
    avg_feed_volume = (feed_volume_sum / total_patients) if total_patients > 0 else 0.0
    avg_fio2 = (fio2_sum / total_patients) if total_patients > 0 else 0.0

    # Prepare the summary data to send as JSON
    summary_data = {
        "total_patients": total_patients,
        "referred_patients": referred_patients,
        "avg_feed_volume": round(avg_feed_volume, 2),
        "avg_fio2": round(avg_fio2, 2)
    }

    return jsonify(summary_data)



if __name__ == '__main__':
    # Create tables if they don't exist
    with app.app_context():
        db.create_all()
    app.run(debug=True)